/*****************************************************************************

        RectalAnarchy.h
        By Laurent de Soras, 2001

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if ! defined (RectalAnarchy_HEADER_INCLUDED)
#define	RectalAnarchy_HEADER_INCLUDED

#if defined (_MSC_VER)
	#pragma once
	#pragma warning (4 : 4250)
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"basic/Array.h"
#include	"dsp/Lfo.h"
#include	"dsp/ParamInertia.h"
#include	"MachineInterface.h"
#include	"RectalAnarchyTrack.h"
#include	"RectalAnarchyChannel.h"



class RectalAnarchy
:	public CMachineInterface
{

/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

public:

	enum {			BUZZ_SAMPLE_SCALE		= 32768	};

	enum {			MCH_VERSION				= 1	};
	enum {			NBR_GLOB_PARAM			= 13	};
	enum {			NBR_TRACK_PARAM		= 3	};
	enum {			NBR_ATTRIBUTES			= 3	};
	enum {			MIN_NBR_TRACKS			= 1	};
	enum {			MAX_NBR_TRACKS			= 16	};
	enum {			MAX_POLYPHONY			= MAX_NBR_TRACKS * 2	};

	enum {			NBR_VOWELS_PER_VOICE	= 5	};
	enum {			NBR_VOICES				= 5	};
	enum {			NBR_TRANSITIONS		= 10	};

	enum
	{
		LFO_SINE = 0,
		LFO_TRIANGLE,
		LFO_SQUARE,
		LFO_RAMPUP,
		LFO_RAMPDOWN,
		LFO_BROWNNOISE,

		NBR_LFO
	};

	struct Formant
	{
		float				_center_freq;		// Formant frequency (Hz)
		float				_bandwidth;			// Formant bandwith (Hz)
		float				_gain;				// Peak volume

		dsp::Filter2p	_filter;				// Used only for computing coefs, not for filtering
	};

						RectalAnarchy ();
	virtual			~RectalAnarchy ();

	// For RectalAnarchyTrack
	int				get_polyphony ();
	RectalAnarchyChannel &
						use_channel (int chn);

	// For RectalAnarchyChannel
	float *			use_adsr_temp_buf ();
	const Formant&	get_formant_data (int fmt) const;

	// CMachineInterface
	virtual void	Init (CMachineDataInput * const pi);
	virtual void	Tick ();
	virtual bool	Work (float *psamples, int numsamples, int const mode);
	virtual void	Stop ();
	virtual void	Save (CMachineDataOutput * const po);
	virtual void	AttributesChanged ();
	virtual void	SetNumTracks (int const n);
	virtual void	MuteTrack (int const i);
	virtual void	MidiNote (int const channel, int const value, int const velocity);
	virtual void	Event (dword const data);
	virtual const char *
						DescribeValue (int const param, int const value);

	// Buzz stuff
	static const CMachineInfo
						_mac_info;



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

protected:



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

#pragma pack (push, 1)	// <- don't forget that

	class GVals
	{
	public:
		word				_glide_time;
		word				_voice_type;
		word				_voice_sound;
		byte				_voice_lfo_depth;
		byte				_voice_lfo_per;
		byte				_voice_lfo_type;
		byte				_emphasis;
		byte				_glottal_open_time;
		byte				_vca_at;
		byte				_vca_dt;
		byte				_vca_sl;
		byte				_vca_st;
		byte				_vca_rt;
	};

	class TVals
	{
	public:
		byte				_note;
		byte				_velo;
		byte				_slide;
	};

	class AVals				// Only int here
	{
	public:
		int				_sound_quality;
		int				_polyphony;
		int				_midi_channel;
	};

#pragma pack (pop)

	class VowelData
	{
	public:
		float				_freq [RectalAnarchyChannel::NBR_FORMANTS];
		float				_gain_db [RectalAnarchyChannel::NBR_FORMANTS];
		float				_bandwidth [RectalAnarchyChannel::NBR_FORMANTS];
	};

	class VoiceData
	{
	public:
		char				_name_0 [19+1];
		VowelData		_vowel [NBR_VOWELS_PER_VOICE];	// A, E, I, O, U
	};

	class GlobalParameters
	{
	public:
		float			_glide_time;
		float			_voice_type;
		float			_voice_sound;
		float			_voice_lfo_depth;
		float			_voice_lfo_per;
		dsp::Lfo::WfType
						_voice_lfo_type;
		float			_emphasis;
		float			_glottal_open_time;
		float			_vca_at;
		float			_vca_dt;
		float			_vca_sl;
		float			_vca_st;
		float			_vca_rt;
	};

	float				buzz_2_note (int param) const;
	float				buzz_2_velo (int param) const;
	float				buzz_2_slide (int param) const;
	float				buzz_2_glide_time (int param) const;
	float				buzz_2_voice_type (int param) const;
	float				buzz_2_voice_sound (int param) const;
	float				buzz_2_voice_lfo_depth (int param) const;
	float				buzz_2_voice_lfo_per (int param) const;
	dsp::Lfo::WfType
						buzz_2_voice_lfo_type (int param) const;
	float				buzz_2_emphasis (int param) const;
	float				buzz_2_glottal_open_time (int param) const;
	float				buzz_2_vca_at (int param) const;
	float				buzz_2_vca_dt (int param) const;
	float				buzz_2_vca_sl (int param) const;
	float				buzz_2_vca_st (int param) const;
	float				buzz_2_vca_rt (int param) const;

	void				set_glide_time (float t);
	void				set_voice_type (float val);
	void				set_voice_sound (float val);
	void				set_voice_lfo_depth (float val);
	void				set_voice_lfo_per (float val);
	void				set_voice_lfo_type (dsp::Lfo::WfType val);
	void				set_emphasis (float val);
	void				set_glottal_open_time (float t);
	void				set_vca_at (float t);
	void				set_vca_dt (float t);
	void				set_vca_sl (float lvl);
	void				set_vca_st (float t);
	void				set_vca_rt (float t);

	void				setup_formant_filters ();
	void				stop_all_notes ();
	void				stop_all_channels ();

	float				_sample_freq;				// Sample frequency, in Hz. > 0
	char				_txt_0 [63+1];				// String used by DescribeValue ()
	int				_nbr_tracks;				// -1 = not init
	int				_nbr_channels;				// -1 = not init
	basic::Array <RectalAnarchyTrack, MAX_NBR_TRACKS>
						_tracks;
	basic::Array <RectalAnarchyChannel, MAX_POLYPHONY>
						_channels;
	float				_adsr_temp_buf [MAX_BUFFER_LENGTH];	// For RectalAnarchyChannel

	GlobalParameters
						_global_param;
	bool				_need_formant_update_flag;
	float				_filter_gain;
	dsp::ParamInertia
						_voice_type;
	dsp::ParamInertia
						_voice_sound;
	dsp::Lfo			_voice_lfo;
	Formant			_formants [RectalAnarchyChannel::NBR_FORMANTS];

	int				_cur_midi_track;
	int				_midi_channel;				// -1 = listen to all channels

	static const char
						COPYRIGHT_0 [];
	static const char *
						VOWEL_NAME_0 [NBR_VOWELS_PER_VOICE];
	static const VoiceData
						VOICE_LIST [NBR_VOICES];
	static const int
						TRANSITION_LIST [NBR_TRANSITIONS];

	// Buzz stuff
	GVals				_gval;
	TVals				_tval [MAX_NBR_TRACKS];
	AVals				_aval;

	static const CMachineParameter			//
						_param_glide_time;
	static const CMachineParameter			// Voice type -> [0 ; 4[
						_param_voice_type;
	static const CMachineParameter			// Voice sound -> [0 ; 4[
						_param_voice_sound;
	static const CMachineParameter			//
						_param_voice_lfo_depth;
	static const CMachineParameter			//
						_param_voice_lfo_per;
	static const CMachineParameter			//
						_param_voice_lfo_type;
	static const CMachineParameter			//
						_param_emphasis;
	static const CMachineParameter			//
						_param_glottal_open_time;
	static const CMachineParameter			//
						_param_vca_at;
	static const CMachineParameter			//
						_param_vca_dt;
	static const CMachineParameter			//
						_param_vca_sl;
	static const CMachineParameter			//
						_param_vca_st;
	static const CMachineParameter			//
						_param_vca_rt;

	static const CMachineParameter			// Note
						_param_note;
	static const CMachineParameter			// Velocity
						_param_velo;
	static const CMachineParameter			// Note slide time
						_param_slide;

	static const CMachineParameter *
						_param_list [NBR_GLOB_PARAM + NBR_TRACK_PARAM];

	static const CMachineAttribute
						_attrib_sound_quality;
	static const CMachineAttribute
						_attrib_polyphony;
	static const CMachineAttribute
						_attrib_midi_channel;

	static const CMachineAttribute *
						_attrib_list [NBR_ATTRIBUTES];



/*\\\ FORBIDDEN MEMBER FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

						RectalAnarchy (const RectalAnarchy &other);
	RectalAnarchy&	operator = (const RectalAnarchy &other);
	bool				operator == (const RectalAnarchy &other) const;
	bool				operator != (const RectalAnarchy &other) const;

};	// class RectalAnarchy



//#include	"RectalAnarchy.hpp"



#endif	// RectalAnarchy_HEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
