/*****************************************************************************

        RectalAnarchyChannel.h
        By Laurent de Soras, 2001

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if ! defined (RectalAnarchyChannel_HEADER_INCLUDED)
#define	RectalAnarchyChannel_HEADER_INCLUDED

#if defined (_MSC_VER)
	#pragma once
	#pragma warning (4 : 4250)
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"dsp/AdsrLog.h"
#include	"dsp/Filter2p.h"
#include	"dsp/Shelf.h"



class RectalAnarchy;
class RectalAnarchyTrack;

class RectalAnarchyChannel
{

/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

public:

	enum {			NBR_FORMANTS			= 5	};

						RectalAnarchyChannel ();
	virtual			~RectalAnarchyChannel ();

	void				init (RectalAnarchy &mch, float sample_freq);
	bool				generate (float data [], long nbr_spl, bool mix_flag);
	bool				is_active () const;
	void				deactivate ();
	double			get_activity_rate () const;

	void				associate_with_track (RectalAnarchyTrack &track);
	void				dissociate_from_track ();
	void				note_on (float midi_note, float slide_time);
	void				note_off ();

	void				set_velo (float velo);

	void				set_glottal_open_time (float t);
	void				set_vca_at (float t);
	void				set_vca_dt (float t);
	void				set_vca_sl (float lvl);
	void				set_vca_st (float t);
	void				set_vca_rt (float t);

	void				set_update_flag ();



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

protected:



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

	void				update_filters ();
	void				generate_voice (float data [], long nbr_spl);
	void				glotal_resync ();

	RectalAnarchy*	_mch_ptr;
	RectalAnarchyTrack *
						_track_ptr;				// Track this channel belongs to
	float				_sample_freq;
	float				_velo;
	float				_glottal_open_time;	// sec, about 1 ms.

	bool				_update_flag;			// Indicates we need update for filter coefs

	dsp::AdsrLog <50000>						// 0.05
						_vol_env;
	double			_cur_freq;
	double			_phase;					// In cycles / sample, [0 ; 1[
	double			_step;					// Current step, in cycles / sample
	double			_slide_speed_log;		// Log of step multiplier for every sample
	double			_slide_final_step;	// Final step
	long				_glotal_len_spl;		// Open glotal phase length, in samples
	long				_glotal_pos;			// Position in sample in the glotal phase
	float				_glotal_val;			// Value for open glotal phase
	float				_glotal_inc_1;			// Value increment
	float				_glotal_inc_2;			// Increment of increment
	dsp::Shelf		_glotal_lpf;			// Filter the glotal waveform
	dsp::Filter2p	_filters [NBR_FORMANTS];



/*\\\ FORBIDDEN MEMBER FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

						RectalAnarchyChannel (const RectalAnarchyChannel &other);
	RectalAnarchyChannel &
						operator = (const RectalAnarchyChannel &other);
	bool				operator == (const RectalAnarchyChannel &other) const;
	bool				operator != (const RectalAnarchyChannel &other) const;

};	// class RectalAnarchyChannel



//#include	"RectalAnarchyChannel.hpp"



#endif	// RectalAnarchyChannel_HEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
