/*****************************************************************************

        RectalAnarchyTrack.cpp
        By Laurent de Soras, 2001

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if defined (_MSC_VER)
	#pragma warning (1 : 4130 4223 4705 4706)
	#pragma warning (4 : 4355 4786 4800)
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"basic/basic_fnc.h"
#include	"RectalAnarchy.h"
#include	"RectalAnarchyTrack.h"

#include	<cassert>



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



RectalAnarchyTrack::RectalAnarchyTrack ()
:	_mch_ptr (0)
,	_sample_freq (44100)
,	_channel_ptr (0)
,	_midi_note (-1)
{
}



RectalAnarchyTrack::~RectalAnarchyTrack ()
{
}



void	RectalAnarchyTrack::init (RectalAnarchy &mch, float sample_freq)
{
	assert (&mch != 0);
	assert (sample_freq > 0);
	assert (_mch_ptr == 0);

	_mch_ptr = &mch;
	_sample_freq = sample_freq;
	_midi_note = -1;
}



void	RectalAnarchyTrack::note_on (float midi_note, float slide_time)
{
	// New note
	if (slide_time < 0)
	{
		note_off ();
		allocate_channel ();
	}

	if (_channel_ptr != 0)
	{
		_channel_ptr->set_velo (_track_param._velo);

		_channel_ptr->note_on (midi_note, slide_time);
		_midi_note = basic::round_int (midi_note);
	}
}



void	RectalAnarchyTrack::note_off ()
{
	if (_channel_ptr != 0)
	{
		_channel_ptr->note_off ();
		_midi_note = -1;
	}
}



int	RectalAnarchyTrack::get_midi_note () const
{
	return (_midi_note);
}



void	RectalAnarchyTrack::dissociate_channel ()
{
	_channel_ptr = 0;
	_midi_note = -1;
}



void	RectalAnarchyTrack::set_velo (float velo)
{
	assert (velo >= 0);

	_track_param._velo = velo;
	if (_channel_ptr != 0)
	{
		_channel_ptr->set_velo (_track_param._velo);
	}
}



void	RectalAnarchyTrack::load (CMachineDataInput * const pi)
{
	if (pi != 0)
	{
		TrackParameters	track_val;
		pi->Read (&track_val, sizeof (track_val));

		set_velo (track_val._velo);
	}
}



void	RectalAnarchyTrack::save (CMachineDataOutput *const po)
{
	assert (po != 0);

	po->Write (&_track_param, sizeof (_track_param));
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



void	RectalAnarchyTrack::allocate_channel ()
{
	int				chn;
	const int		nbr_chn = _mch_ptr->get_polyphony ();
	int				selected_chn = -1;

	for (chn = 0; chn < nbr_chn; ++chn)
	{
		if (! _mch_ptr->use_channel (chn).is_active ())
		{
			selected_chn = chn;
			break;
		}
	}

	if (selected_chn < 0)
	{
		double			min_rate = 1;
		for (chn = 0; chn < nbr_chn; ++chn)
		{
			const double	rate = _mch_ptr->use_channel (chn).get_activity_rate ();
			if (rate < min_rate)
			{
				selected_chn = chn;
				min_rate = rate;
			}
		}
	}

	if (selected_chn >= 0)
	{
		// Order of these instructions is important (_channel_ptr modification
		// in callback routine).
		RectalAnarchyChannel &	channel = _mch_ptr->use_channel (selected_chn);
		channel.note_off ();
		channel.associate_with_track (*this);
		_channel_ptr = &channel;
	}
}



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
