/*****************************************************************************
*                                                                            *
*       PROJECT NAME                                                         *
*       Version 1.00, 2001/01/01                                             *
*       (c) Ohm Force                                                        *
*                                                                            *
*       AdsrLog.h                                                            *
*       Short file description                                               *
*       Portability/Compatibility                                            *
*                                                                            *
* Tab = 3                                                                    *
*****************************************************************************/



#if defined (dsp_AdsrLog_CURRENT_HEADER)
	#error Recursive inclusion of dsp/AdsrLog header.
#endif
#define	dsp_AdsrLog_CURRENT_HEADER

#if ! defined (dsp_AdsrLog_HEADER_INCLUDED)
#define	dsp_AdsrLog_HEADER_INCLUDED

#if defined (_MSC_VER)
	#pragma warning (4 : 4250) // "Inherits via dominance."
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"dsp/AdsrBase.h"



namespace dsp
{



template <long ZERO_LEVEL = 50000>	// Scale : 1e6. (50000 => -26 dB)
class AdsrLog
:	public AdsrBase
{

/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

public:

						AdsrLog ();

	// AdsrBase
	void				set_sample_freq (float fs);
	void				set_attack_time (float at);
	void				set_decay_time (float dt);
	void				set_sustain_level (float sl);
	void				set_sustain_time (float st);
	void				set_release_time (float rt);
	inline AdsrBase::Section
						get_cur_section () const;
	void				note_on ();
	void				note_off ();

	void				clear_buffers ();
	void				kill_ramps ();

	inline float	get_val () const;
	inline float	process_sample ();
	void				process_block (float data [], long nbr_spl);
	void				process_block_mul (float data [], long nbr_spl);
	void				skip_block (long nbr_spl);



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

protected:



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

	void				set_next_section ();

	float				_sample_freq;		// Hz
	float				_attack_time;		// s, >= 0
	float				_decay_time;
	float				_sustain_level;	// [0 ; 1]
	float				_sustain_time;
	float				_release_time;

	double			_val;
	double			_offset;				// _val + _offset [0 ; 1]
	double			_step;
	long				_nbr_rem_spl;		// >= 0
	Section			_section;



/*\\\ FORBIDDEN MEMBER FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

						AdsrLog (const AdsrLog &other);
	AdsrLog &		operator = (const AdsrLog &other);
	bool				operator == (const AdsrLog &other);
	bool				operator != (const AdsrLog &other);

};	// class AdsrLog

}	// namespace dsp



/*\\\ GLOBAL OPERATOR PROTOTYPES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ CODE HEADER INCLUSION \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"dsp/AdsrLog.hpp"



#endif	// dsp_AdsrLog_HEADER_INCLUDED

#undef dsp_AdsrLog_CURRENT_HEADER



/*****************************************************************************

	LEGAL

	(c) 2001 Ohm Force

	Use of this file or part of its content without written permission is
	strictly forbidden.
	For more information, please contact Ohm Force at :
	contact@ohmforce.com

*****************************************************************************/



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
