/*****************************************************************************
*                                                                            *
*       PROJECT NAME                                                         *
*       Version 1.00, 2001/01/03                                             *
*       (c) Ohm Force                                                        *
*                                                                            *
*       AdsrLog.hpp                                                          *
*       Short file description                                               *
*       Portability/Compatibility                                            *
*                                                                            *
* Tab = 3                                                                    *
*****************************************************************************/



#if defined (dsp_AdsrLog_CURRENT_CODEHEADER)
	#error Recursive inclusion of AdsrLog code header.
#endif
#define	dsp_AdsrLog_CURRENT_CODEHEADER

#if ! defined (dsp_AdsrLog_CODEHEADER_INCLUDED)
#define	dsp_AdsrLog_CODEHEADER_INCLUDED



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	<cassert>
#include	<climits>



namespace dsp
{



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



template <long ZERO_LEVEL>
AdsrLog <ZERO_LEVEL>::AdsrLog ()
:	_sample_freq (44100)
,	_attack_time (0.01f)
,	_decay_time (0.1f)
,	_sustain_level (0.85f)
,	_sustain_time (4.0f)
,	_release_time (0.25f)
{
	clear_buffers ();
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::set_sample_freq (float fs)
{
	assert (fs > 0);
   assert (fs <= MAX_SAMPLE_FREQ);

	_sample_freq = fs;
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::set_attack_time (float at)
{
	assert (at >= 0);
	assert (at * _sample_freq <= LONG_MAX);
	_attack_time = at;
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::set_decay_time (float dt)
{
	assert (dt >= 0);
	assert (dt * _sample_freq <= LONG_MAX);
	_decay_time = dt;
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::set_sustain_level (float sl)
{
	assert (sl >= 0);
	assert (sl <= 1);
	_sustain_level = sl;
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::set_sustain_time (float st)
{
	assert (st >= 0);
	assert (st * _sample_freq <= LONG_MAX);
	_sustain_time = st;
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::set_release_time (float rt)
{
	assert (rt >= 0);
	assert (rt * _sample_freq <= LONG_MAX);
	_release_time = rt;
}



template <long ZERO_LEVEL>
AdsrBase::Section	AdsrLog <ZERO_LEVEL>::get_cur_section () const
{
	return (_section);
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::note_on ()
{
	_offset = 0;
	_nbr_rem_spl = long (_attack_time * _sample_freq);
	_nbr_rem_spl = basic::max (_nbr_rem_spl, 1L);
	_step = (1.0 - _val) / _nbr_rem_spl;
	_section = Section_ATTACK;
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::note_off ()
{
	if (   _section != Section_NONE
	    && _section != Section_RELEASE)
	{
		_section = Section_SUSTAIN;
		set_next_section ();
	}
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::clear_buffers ()
{
	_val = 0;
	_offset = 0;
	_step = 0;
	_nbr_rem_spl = LONG_MAX;
	_section = Section_NONE;
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::kill_ramps ()
{
	if (   _section == Section_NONE
	    || (_section == Section_ATTACK && _offset == 0))
	{
		_val = 0;
		_step = 1.0 / _nbr_rem_spl;
	}
}



template <long ZERO_LEVEL>
float	AdsrLog <ZERO_LEVEL>::get_val () const
{
	assert (float (_val + _offset) >= 0);
//	assert (float (_val + _offset) <= 1);		// May be slightly higher than 1 because of rounding errors

	return (float (_val + _offset));
}



template <long ZERO_LEVEL>
float	AdsrLog <ZERO_LEVEL>::process_sample ()
{
	if (_section == Section_ATTACK)
	{
		_val += _step;
	}
	else
	{
		_val *= _step;
	}

	--_nbr_rem_spl;
	if (_nbr_rem_spl <= 0)
	{
		set_next_section ();
	}

	return (get_val ());
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::process_block (float data [], long nbr_spl)
{
	assert (data != 0);
	assert (nbr_spl > 0);

	long				pos = 0;
	do
	{
		/*** A optimiser !!! ***/

		data [pos] = process_sample ();
		++pos;
	}
	while (pos < nbr_spl);
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::process_block_mul (float data [], long nbr_spl)
{
	assert (data != 0);
	assert (nbr_spl > 0);

	long				pos = 0;
	do
	{
		/*** A optimiser !!! ***/

		data [pos] *= process_sample ();
		++pos;
	}
	while (pos < nbr_spl);
}



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::skip_block (long nbr_spl)
{
	assert (nbr_spl > 0);

	long				pos = 0;
	do
	{
		/*** A optimiser !!! ***/

		process_sample ();
		++pos;
	}
	while (pos < nbr_spl);
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



template <long ZERO_LEVEL>
void	AdsrLog <ZERO_LEVEL>::set_next_section ()
{
	using namespace std;

	static const double	log_zero_level = log (ZERO_LEVEL * 1e-6);

	switch (_section)
	{
	case	Section_ATTACK:
		_section = Section_DECAY;
		_nbr_rem_spl = long (_decay_time * _sample_freq);
		_nbr_rem_spl = basic::max (_nbr_rem_spl, 1L);
		_val = (1 - _sustain_level) * (1.0 / (1 - ZERO_LEVEL * 1e-6));
		_offset = 1 - _val;
		_step = exp (log_zero_level / _nbr_rem_spl);
		break;

	case	Section_DECAY:
		_section = Section_SUSTAIN;
		_nbr_rem_spl = long (_sustain_time * _sample_freq);
		_nbr_rem_spl = basic::max (_nbr_rem_spl, 1L);
		if (_sustain_time >= INFINITE_SUSTAIN - 1)	// -1 to avoid rounding errors.
		{
			_val = 0;
			_offset = _sustain_level;
			_step = 0;
		}
		else
		{
			_val = _sustain_level * (1.0 / (1 - ZERO_LEVEL * 1e-6));
			_offset = _sustain_level - _val;
			_step = exp (log_zero_level / _nbr_rem_spl);
		}
		break;

	case	Section_SUSTAIN:
		{
			const double	old_val = _val + _offset;
			if (basic::is_null (old_val))
			{
				clear_buffers ();
			}
			else
			{
				_section = Section_RELEASE;
				_nbr_rem_spl = long (_release_time * _sample_freq);
				_nbr_rem_spl = basic::max (_nbr_rem_spl, 1L);
				_val = old_val * (1.0 / (1 - ZERO_LEVEL * 1e-6));
				_offset = old_val - _val;
				_step = exp (log_zero_level / _nbr_rem_spl);
			}
		}
		break;

	case	Section_RELEASE:
		clear_buffers ();
		break;

	case	Section_NONE:
		_nbr_rem_spl = LONG_MAX;
		break;

	default:
		assert (false);
		break;
	}
}



}	// namespace dsp



/*\\\ GLOBAL OPERATOR DEFINITIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



#endif	// dsp_AdsrLog_CODEHEADER_INCLUDED

#undef dsp_AdsrLog_CURRENT_CODEHEADER



/*****************************************************************************

	LEGAL

	(c) 2001 Ohm Force

	Use of this file or part of its content without written permission is
	strictly forbidden.
	For more information, please contact Ohm Force at :
	contact@ohmforce.com

*****************************************************************************/



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
