/*****************************************************************************

        Biquad.h

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if ! defined (dsp_Biquad_HEADER_INCLUDED)
#define	dsp_Biquad_HEADER_INCLUDED

#if defined (_MSC_VER)
	#pragma once
	#pragma warning (4 : 4250)
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



namespace dsp
{



class EnvFollowerBase;

// Can be inherited but is not polymorph.
class Biquad
{

/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

public:

						Biquad ();

	inline void		set_z_eq (const float b [3], const float a [3]);
	inline void		set_mem_x (int delay, float val);
	inline float	get_mem_x (int delay);
	inline void		set_mem_y (int delay, float val);
	inline float	get_mem_y (int delay);
	inline void		apply_volume (float gain);
	inline void		copy_filter (const Biquad &other);

	inline float	process_sample (float x);
	inline float	process_sample_nd (float x);
	inline float	process_sample_clip (float x, float limit = 1.0f);
	inline float	process_sample_limit (float x, EnvFollowerBase &env_follower, float inv_limit = 1.0f);

	void				process_block (float *dest_ptr, const float *src_ptr, long nbr_spl);
	void				process_block (float *spl_ptr, long nbr_spl);
	void				process_block_mix (float *dest_ptr, const float *src_ptr, long nbr_spl);
	void				process_block_mix (float *spl_ptr, long nbr_spl);
	void				process_block_nd (float *dest_ptr, const float *src_ptr, long nbr_spl);
	void				process_block_nd (float *spl_ptr, long nbr_spl);
	void				process_block_clip (float *dest_ptr, const float *src_ptr, long nbr_spl, float limit = 1.0f);
	void				process_block_clip (float *spl_ptr, long nbr_spl, float limit = 1.0f);
	void				process_block_limit (float *dest_ptr, const float *src_ptr, long nbr_spl, EnvFollowerBase &env_follower, float inv_limit = 1.0f);
	void				process_block_limit (float *spl_ptr, long nbr_spl, EnvFollowerBase &env_follower, float inv_limit = 1.0f);

   void           set_mem_dc (float dc);
	void				clip_memory (float limit = 1.0f);
	inline void		clip_memory_fast (float limit = 1.0f);
	void				clear_buffers ();

	bool				check_denormal () const;
	void				kill_denormal ();



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

protected:

	float				_z_eq_b [3];	// Direct coefficients, order z^(-n)
	float				_z_eq_a [3];	// Recursive coefficients, order z^(-n)



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

	float				_mem_x [2];		// Input memory, order z^(-n)
	float				_mem_y [2];		// Output memory, order z^(-n)
	int				_mem_pos;		// 0 or 1



/*\\\ FORBIDDEN MEMBER FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

						Biquad (const Biquad &other);
	Biquad &			operator = (const Biquad &other);
	bool				operator == (const Biquad &other) const;
	bool				operator != (const Biquad &other) const;

};	// class Biquad



}	// namespace dsp



#include	"dsp/Biquad.hpp"



#endif	// dsp_Biquad_HEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
