/*****************************************************************************

        EnvFollowerBase.cpp

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if defined (_MSC_VER)
	#pragma warning (1 : 4130) // "'operator' : logical operation on address of string constant"
	#pragma warning (1 : 4223) // "nonstandard extension used : non-lvalue array converted to pointer"
	#pragma warning (1 : 4705) // "statement has no effect"
	#pragma warning (1 : 4706) // "assignment within conditional expression"
	#pragma warning (4 : 4786) // "identifier was truncated to '255' characters in the debug information"
	#pragma warning (4 : 4800) // "forcing value to bool 'true' or 'false' (performance warning)"
	#pragma warning (4 : 4355) // "'this' : used in base member initializer list"
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"basic/basic_fnc.h"
#include	"dsp/EnvFollowerBase.h"



namespace dsp
{



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



EnvFollowerBase::EnvFollowerBase ()
:	_state (0)
,	_attack_coef (0.125f)
,	_release_coef (0.125f)
{
}



void	EnvFollowerBase::process_block (float dest_ptr [], const float src_ptr [], long nbr_spl)
{
	assert (dest_ptr != 0);
	assert (src_ptr != 0);
	assert (nbr_spl > 0);

	const float		attack_coef = _attack_coef;
	const float		release_coef = _release_coef;
	float				state = _state;

	long				pos = 0;
	do
	{
		using namespace std;
		const float		delta = float (fabs (src_ptr [pos]) - state);
		const float		coef = (delta > 0) ? attack_coef : release_coef;

		state += delta * coef;
		assert (state >= 0);
		dest_ptr [pos] = state;

		++pos;
	}
	while (pos < nbr_spl);

	_state = state;

	clear_denormal ();
}



float	EnvFollowerBase::analyse_block (const float spl_ptr [], long nbr_spl)
{
	assert (spl_ptr != 0);
	assert (nbr_spl > 0);

	const float		attack_coef = _attack_coef;
	const float		release_coef = _release_coef;
	float				state = _state;

	long				pos = 0;
	do
	{
		using namespace std;
		const float		delta = float (fabs (spl_ptr [pos]) - state);
		const float		coef = (delta > 0) ? attack_coef : release_coef;

		state += delta * coef;
		assert (state >= 0);

		++pos;
	}
	while (pos < nbr_spl);

	_state = state;

	clear_denormal ();

	return (state);
}



void	EnvFollowerBase::skip_constant_block (float cst_in, long nbr_spl)
{
	assert (nbr_spl > 0);

	using namespace std;

	const float		delta = float (fabs (cst_in) - _state);
	const float		coef = (delta > 0) ? _attack_coef : _release_coef;
	_state += (1 - exp (log (1 - coef) * nbr_spl)) * delta;

	clear_denormal ();
}



void	EnvFollowerBase::clear_buffers ()
{
	_state = 0;
}



void	EnvFollowerBase::clear_denormal ()
{
	if (! check_denormal ())
	{
		_state = 0;
	}
}



/*
==============================================================================
Name : check_denormal
Description :
	Check if there is no denormal number in the envelope follower recursive
	memory. This function is designed to be directly used in assert().
Returns: true if there isn't any denormal number.
Throws : Nothing
==============================================================================
*/

bool	EnvFollowerBase::check_denormal () const
{
	return (! basic::is_denormal (_state));
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ GLOBAL OPERATOR DEFINITIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

}	// namespace dsp



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
