/*****************************************************************************

        EnvFollowerBase.h

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if ! defined (dsp_EnvFollowerBase_HEADER_INCLUDED)
#define	dsp_EnvFollowerBase_HEADER_INCLUDED

#if defined (_MSC_VER)
	#pragma once
	#pragma warning (4 : 4250) // "Inherits via dominance."
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



namespace dsp
{



// Can be inherited but is not polymorph.
class EnvFollowerBase
{

/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

public:

						EnvFollowerBase ();

	inline void		set_state (float state);
	inline float	get_state () const;
	inline void		set_attack_coef (float coef);
	inline float	get_attack_coef () const;
	inline void		set_release_coef (float coef);
	inline float	get_release_coef () const;

	inline float	process_sample (float x);
	void				process_block (float dest_ptr [], const float src_ptr [], long nbr_spl);
	float				analyse_block (const float spl_ptr [], long nbr_spl);
	void				skip_constant_block (float cst_in, long nbr_spl);
	inline void		apply_volume (float gain);

	void				clear_buffers ();

	void				clear_denormal ();
	bool				check_denormal () const;



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

protected:

	float				_state;					// abs (Sample), >= 0
	float				_attack_coef;			// [0 ; 1]
	float				_release_coef;			// [0 ; 1]



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:



/*\\\ FORBIDDEN MEMBER FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

						EnvFollowerBase (const EnvFollowerBase &other);
	EnvFollowerBase &
						operator = (const EnvFollowerBase &other);
	bool				operator == (const EnvFollowerBase &other);
	bool				operator != (const EnvFollowerBase &other);

};	// class EnvFollowerBase



/*\\\ GLOBAL OPERATOR PROTOTYPES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

}	// namespace dsp



/*\\\ CODE HEADER INCLUSION \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"dsp/EnvFollowerBase.hpp"



#endif	// dsp_EnvFollowerBase_HEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
