/*****************************************************************************

        EnvFollowerBase.hpp

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if defined (dsp_EnvFollowerBase_CURRENT_CODEHEADER)
	#error Recursive inclusion of EnvFollowerBase code header.
#endif
#define	dsp_EnvFollowerBase_CURRENT_CODEHEADER

#if ! defined (dsp_EnvFollowerBase_CODEHEADER_INCLUDED)
#define	dsp_EnvFollowerBase_CODEHEADER_INCLUDED



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	<cassert>
#include	<cmath>



namespace std { }

namespace dsp
{



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



void	EnvFollowerBase::set_state (float state)
{
	assert (state >= 0);

	_state = state;
}



float	EnvFollowerBase::get_state () const
{
	return (_state);
}



void	EnvFollowerBase::set_attack_coef (float coef)
{
	assert (coef >= 0);
	assert (coef <= 1);

	_attack_coef = coef;
}



float	EnvFollowerBase::get_attack_coef () const
{
	return (_attack_coef);
}



void	EnvFollowerBase::set_release_coef (float coef)
{
	assert (coef >= 0);
	assert (coef <= 1);

	_release_coef = coef;
}



float	EnvFollowerBase::get_release_coef () const
{
	return (_release_coef);
}



float	EnvFollowerBase::process_sample (float x)
{
	using namespace std;

	const float		delta = float (fabs (x) - _state);
	const float		coef = (delta > 0) ? _attack_coef : _release_coef;

	_state += delta * coef;
	assert (_state >= 0);

	return (_state);
}



void	EnvFollowerBase::apply_volume (float gain)
{
	assert (gain >= 0);

	_state *= gain;
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ GLOBAL OPERATOR DEFINITIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

}	// namespace dsp



#endif	// dsp_EnvFollowerBase_CODEHEADER_INCLUDED

#undef dsp_EnvFollowerBase_CURRENT_CODEHEADER



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
