/*****************************************************************************

        FastRand.hpp

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if ! defined (dsp_FastRand_CODEHEADER_INCLUDED)
#define	dsp_FastRand_CODEHEADER_INCLUDED



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"basic/basic_fnc.h"

#include	<cassert>



namespace dsp
{



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*
==============================================================================
Name: ctor
Throws: Nothing
==============================================================================
*/

FastRand::FastRand ()
{
	clear_buffers ();
}



/*
==============================================================================
Name: get_new_val
Description:
	Computes a new random value.
Returns: The new value.
Throws: Nothing
==============================================================================
*/

long	FastRand::get_new_val ()
{
	_current_val = (_current_val * A_VAL + B_VAL) & M_VAL;

	return (_current_val);
}



long	FastRand::get_new_val (long nbr_val)
{
	assert (nbr_val > 0);

	const float		x = float (get_new_val ()) * nbr_val;
	const long		val = basic::floor_int (x / (float (M_VAL) + 1));

	return (val);
}



void	FastRand::set_seed (long val)
{
	assert (val != 0);

	_current_val = val;
}



void	FastRand::set_seed_natural ()
{
	const long		val = long ((ptrdiff_t) (this));
	set_seed (val);
}



/*
==============================================================================
Name: clear_buffers
Description:
	Resets the random generator and sets the seed to 1.
Throws: Nothing
==============================================================================
*/

void	FastRand::clear_buffers ()
{
	_current_val = 1;
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



}	// namespace dsp



#endif	// dsp_FastRand_CODEHEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
