/*****************************************************************************

        Filter2p.cpp

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if defined (_MSC_VER)
	#pragma warning (1 : 4130 4223 4705 4706)
	#pragma warning (4 : 4355 4786 4800)
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"dsp/Filter2p.h"

#include	<cassert>



namespace dsp
{



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



const char *const	Filter2p::TYPE_NAME_0 [NBR_TYPES] =
{
	"Low-pass", "High-pass", "Resonator", "Band-pass"
};



/*
==============================================================================
Name: ctor
Throws: Nothing
==============================================================================
*/

Filter2p::Filter2p ()
:	_type (RESONATOR)
,	_reso (1)
,	_q (1)
{
	update_eq ();
}



/*
==============================================================================
Name: update_eq
Description:
	Modifies the filter equation. Must be called after a parameter change.
Throws: Nothing
==============================================================================
*/

void	Filter2p::update_eq ()
{
	// Transfert function coefficients in s plane
	switch (_type)
	{
	case	LOWPASS:
		_s_eq_b [2] = 0;
		_s_eq_b [1] = 0;
		_s_eq_b [0] = 1;
		_s_eq_a [1] = 1 / _reso;
		break;

	case	HIGHPASS:
		_s_eq_b [2] = 1;
		_s_eq_b [1] = 0;
		_s_eq_b [0] = 0;
		_s_eq_a [1] = 1 / _reso;
		break;

	case	RESONATOR:
		_s_eq_b [2] = 1;
		_s_eq_b [1] = _reso / _q;
		_s_eq_b [0] = 1;
		_s_eq_a [1] = 1 / _q;
		break;

	case	BANDPASS:
		_s_eq_b [2] = 0;
		_s_eq_b [1] = _reso / _q;
		_s_eq_b [0] = 0;
		_s_eq_a [1] = 1 / _q;
		break;

	default:
		assert (false);
		break;
	};

	_s_eq_a [2] = 1;
	_s_eq_a [0] = 1;

	transform_s_to_z ();
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



}	// namespace dsp



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
