/*****************************************************************************

        Filter2p.hpp

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if ! defined (dsp_Filter2p_CODEHEADER_INCLUDED)
#define	dsp_Filter2p_CODEHEADER_INCLUDED



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	<cassert>



namespace dsp
{



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*
==============================================================================
Name: set_type
Description:
	Sets the filter type. update_eq() must be called afterwards in order to
	validate the change.
Input parameters:
	- type: Filter type (see enum).
Throws: Nothing
==============================================================================
*/

void	Filter2p::set_type (Type type)
{
	_type = type;
}



Filter2p::Type	Filter2p::get_type () const
{
	return (_type);
}



/*
==============================================================================
Name: set_reso
Description:
	Sets the resonance. update_eq() must be called afterwards in order to
	validate the change.
Input parameters:
	- reso: Resonance, which is approximately the gain at the cutoff frequency.
Throws: Nothing.
==============================================================================
*/

void	Filter2p::set_reso (float reso)
{
	assert (reso > 0);
	_reso = reso;
}



float	Filter2p::get_reso () const
{
	return (_reso);
}



/*
==============================================================================
Name: set_q
Description:
	Sets the filter quality (only for resonant and band-pass modes).
	update_eq() must be called afterwards in order to validate the change.
Input parameters:
	- q: Quality (inverse of relative bandwidth). The higher q is, the more
		selective the filter is.
Throws: Nothing.
==============================================================================
*/

void	Filter2p::set_q (float q)
{
	assert (q > 0);
	_q = q;
}



float	Filter2p::get_q () const
{
	return (_q);
}



/*
==============================================================================
Name: copy_filter
Description:
	Copies the filter, without its state.
Input parameters:
	- other: Filter to be copied.
Throws: Nothing
==============================================================================
*/

void	Filter2p::copy_filter (const Filter2p &other)
{
	Inherited::copy_filter (other);

	_type = other._type;
	_reso = other._reso;
	_q    = other._q;
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



}	// namespace dsp



#endif	// dsp_Filter2p_CODEHEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
