/*****************************************************************************

        Lfo.h

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if ! defined (dsp_Lfo_HEADER_INCLUDED)
#define	dsp_Lfo_HEADER_INCLUDED

#if defined (_MSC_VER)
	#pragma once
	#pragma warning (4 : 4250)
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"basic/Array.h"
#include	"basic/basic_def.h"
#include	"dsp/FastRand.h"
#include	"dsp/InterpFncPeriodic.h"
#include	"dsp/Shelf.h"



namespace std { }



namespace dsp
{



class Lfo
{

/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

public:

	enum {			NOISE_RES = 256						};

	enum WfType
	{
		SINE = 0,
		TRIANGLE,
		SQUARE,
		RAMP_UP,
		COS_UP,
		RAMP_DOWN,
		COS_DOWN,
		RANDOM,
		BROWN_NOISE,
		RED_NOISE,

		NBR_WF_TYPES
	};

						Lfo ();

	inline void		set_sample_freq (float fs);
	inline float	get_sample_freq () const;
	inline void		set_type (WfType type);
	inline void		set_period (double period);
	inline double	get_period () const;
	inline void		set_amp (float amp);
	inline float	get_amp () const;
	inline void		set_phase (float phase);
	inline float	get_phase () const;

	inline float	get_value () const;
	inline float	get_raw_value () const;

	void				tick (long nbr_spl);

	void				clear_buffers ();

	static const char *const
						WF_TYPE_NAME_0 [NBR_WF_TYPES];



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

protected:



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

	enum {			CYCLE_RES_LOG2		= 4							};
	enum {			CYCLE_RES			= 1 << CYCLE_RES_LOG2	};
	enum {			MAX_FILTER_STAGES	= 2							};

	typedef	basic::Array <float, MAX_FILTER_STAGES>	StateArr;

	struct SinFnc
	{
		double			operator () (double x) const
		{
			using namespace std;
			return (sin (x * 2 * basic::PI));
		}
	};

	void				update_param ();
	inline void		gen_rand_val ();

	WfType			_type;
	float				_amp;				// %
	float				_fs;				// Hz
	double			_period;			// Seconds

	double			_pos;
	double			_step;
	float				_random_val;
	long				_noise_pos;
	float				_filter_amp;

	InterpFncPeriodic <CYCLE_RES_LOG2, SinFnc>
						_sin_gen;
	FastRand			_rand_gen;
	StateArr			_cur_val_arr;
	float				_filter_coef;



/*\\\ FORBIDDEN MEMBER FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

						Lfo (const Lfo &other);
	Lfo &				operator = (const Lfo &other);
	bool				operator == (const Lfo &other) const;
	bool				operator != (const Lfo &other) const;

};	// class Lfo



}	// namespace dsp



#include	"dsp/Lfo.hpp"



#endif	// dsp_Lfo_HEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
