/*****************************************************************************

        OnePole.h

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if ! defined (dsp_OnePole_HEADER_INCLUDED)
#define	dsp_OnePole_HEADER_INCLUDED

#if defined (_MSC_VER)
	#pragma once
	#pragma warning (4 : 4250)
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



namespace dsp
{



// Can be derived but is not polymorph
class OnePole
{

/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

public:

						OnePole ();

	inline void		set_z_eq (const float b [2], const float a [2]);
	inline void		apply_volume (float gain);
	inline void		copy_filter (const OnePole &other);

	inline float	process_sample (float x);
	inline float	process_sample_nd (float x);

	void				process_block (float *dest_ptr, const float *src_ptr, long nbr_spl);
	inline void		process_block (float *spl_ptr, long nbr_spl);
	void				process_block_nd (float *dest_ptr, const float *src_ptr, long nbr_spl);
	inline void		process_block_nd (float *spl_ptr, long nbr_spl);

   void           set_mem_dc (float dc);
	void				clip_memory (float limit = 1.0f);
	inline void		clip_memory_fast (float limit = 1.0f);
	inline float	get_latest_output_sample () const;
	void				clear_buffers ();

	bool				check_denormal () const;
	void				kill_denormal ();



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

protected:

	float				_z_eq_b [2];	// Coefficients directs, ordre z^(-n)
	float				_z_eq_a [2];	// Coefficients recursifs, ordre z^(-n).
	// _z_eq_a [0] n'est jamais utilise; on sous-entend qu'il vaut toujours 1.



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

	float				_mem_x;			// Memoire de l'entree
	float				_mem_y;			// Memoire de la sortie



/*\\\ FORBIDDEN MEMBER FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

						OnePole (const OnePole &other);
	OnePole &		operator = (const OnePole &other);
	bool				operator == (const OnePole &other) const;
	bool				operator != (const OnePole &other) const;

};	// class OnePole



}	// namespace dsp



#include	"dsp/OnePole.hpp"



#endif	// dsp_OnePole_HEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
