/*****************************************************************************

        OnePoleS.cpp

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if defined (_MSC_VER)
	#pragma warning (1 : 4130 4223 4705 4706)
	#pragma warning (4 : 4355 4786 4800)
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"basic/basic_def.h"
#include	"basic/basic_fnc.h"
#include	"dsp/OnePoleS.h"

#include	<cmath>



namespace std { }

namespace dsp
{



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



OnePoleS::OnePoleS ()
:	_sample_freq (44100)
,	_f0 (1000)
{
	_s_eq_a [0] = 1;
	_s_eq_a [1] = 1;

	_s_eq_b [0] = _s_eq_a [0];
	_s_eq_b [1] = _s_eq_a [1];
}



/*
==============================================================================
Name: transform_s_to_z
Description:
	Transforms the continous s-plane equation in a discrete z-plane equation
	which can be used directly to filter signal, using the bilinear transfrom.

        ---        k                  ---        -k
        \   a[k].s                    \   a'[k].z
        /__                           /__
H(s) = --------------   ==>   H(z) = ---------------
        ---       k                   ---        -k
        \   b[k].s                    \   b'[k].z
        /__                           /__

Throws: Nothing
==============================================================================
*/

void	OnePoleS::transform_s_to_z ()
{
	using namespace std;
	using basic::is_null;
	using basic::PI;

	// s to z bilinear transform
	const double	inv_k = tan (_f0 * PI / _sample_freq);
	assert (! is_null (inv_k));
	const double	k = 1 / inv_k;

	const double	b1k = _s_eq_b [1] * k;
	const double	b1z = _s_eq_b [0] - b1k;
	const double	b0z = _s_eq_b [0] + b1k;

	const double	a1k = _s_eq_a [1] * k;
	const double	a1z = _s_eq_a [0] - a1k;
	const double	a0z = _s_eq_a [0] + a1k;

	// IIR coefficients
	assert (! is_null (a0z));
	const double	mult = 1 / a0z;

	_z_eq_b [0] = float (b0z * mult);
	_z_eq_b [1] = float (b1z * mult);

	_z_eq_a [0] = 1;
	_z_eq_a [1] = float (a1z * mult);
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



}	// namespace dsp



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
