/*****************************************************************************

        Shelf.cpp

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if defined (_MSC_VER)
	#pragma warning (1 : 4130 4223 4705 4706)
	#pragma warning (4 : 4355 4786 4800)
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"dsp/Shelf.h"

#include	<cassert>



namespace dsp
{



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*
==============================================================================
Name: ctor
Throws: Nothing
==============================================================================
*/

Shelf::Shelf ()
:	_level (0)
{
	make_high_shelf ();
}



/*
==============================================================================
Name: make_high_shelf
Description:
	Builds the high-boost/cut filter from the current parameters.
Throws: Nothing
==============================================================================
*/

void	Shelf::make_high_shelf ()
{
	assert (get_freq () < get_sample_freq () * 0.5);

	// Transfert function coefficients in s plane
	_s_eq_b [0] = 1;
	_s_eq_b [1] = _level;
	_s_eq_a [0] = 1;
	_s_eq_a [1] = 1;

	transform_s_to_z ();
}



/*
==============================================================================
Name: make_low_shelf
	Builds the low-boost/cut filter from the current parameters.
Throws: Nothing
==============================================================================
*/

void	Shelf::make_low_shelf ()
{
	assert (get_freq () < get_sample_freq () * 0.5);

	// Transfert function coefficients in s plane
	_s_eq_b [0] = _level;
	_s_eq_b [1] = 1;
	_s_eq_a [0] = 1;
	_s_eq_a [1] = 1;

	transform_s_to_z ();
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



}	// namespace dsp



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
